@@ -52,6 +52,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include <highgui.h>
+#include <limits.h>
 
 #ifdef CV_VERBOSE
 #include <time.h>
@@ -1163,7 +1164,8 @@ CvBackgroundData* icvCreateBackgroundData( const char* filename, CvSize winsize
         while( !feof( input ) )
         {
             *imgfilename = '\0';
-            fscanf( input, "%s", imgfilename );
+            if( !fscanf( input, "%s", imgfilename ))
+                break;
             len = strlen( imgfilename );
             if( len > 0 )
             {
@@ -1174,7 +1176,8 @@ CvBackgroundData* icvCreateBackgroundData( const char* filename, CvSize winsize
         }
         if( count > 0 )
         {
-            rewind( input );
+            //rewind( input );
+            fseek( input, 0, SEEK_SET );
             datasize += sizeof( *data ) + sizeof( char* ) * count;
             data = (CvBackgroundData*) cvAlloc( datasize );
             memset( (void*) data, 0, datasize );
@@ -1188,7 +1191,8 @@ CvBackgroundData* icvCreateBackgroundData( const char* filename, CvSize winsize
             while( !feof( input ) )
             {
                 *imgfilename = '\0';
-                fscanf( input, "%s", imgfilename );
+                if( !fscanf( input, "%s", imgfilename ))
+                    break;
                 len = strlen( imgfilename );
                 if( len > 0 )
                 {
@@ -1257,6 +1261,7 @@ void icvGetNextFromBackgroundData( CvBackgroundData* data,
     size_t datasize = 0;
     int round = 0;
     int i = 0;
+    CvPoint offset = cvPoint(0,0);
 
     assert( data != NULL && reader != NULL );
 
@@ -1284,12 +1289,20 @@ void icvGetNextFromBackgroundData( CvBackgroundData* data,
 //#endif /* CV_VERBOSE */
           
             img = cvLoadImage( data->filename[data->last++], 0 );
+            if( !img )
+                continue;
             data->round += data->last / data->count;
             data->round = data->round % (data->winsize.width * data->winsize.height);
             data->last %= data->count;
 
+            offset.x = round % data->winsize.width;
+            offset.y = round / data->winsize.width;
+
+            offset.x = MIN( offset.x, img->width - data->winsize.width );
+            offset.y = MIN( offset.y, img->height - data->winsize.height );
+            
             if( img != NULL && img->depth == IPL_DEPTH_8U && img->nChannels == 1 &&
-                img->width >= data->winsize.width && img->height >= data->winsize.height )
+                offset.x >= 0 && offset.y >= 0 )
             {
                 break;
             }
@@ -1315,8 +1328,9 @@ void icvGetNextFromBackgroundData( CvBackgroundData* data,
     cvReleaseImage( &img );
     img = NULL;
 
-    reader->offset.x = round % data->winsize.width;
-    reader->offset.y = round / data->winsize.width;
+    //reader->offset.x = round % data->winsize.width;
+    //reader->offset.y = round / data->winsize.width;
+    reader->offset = offset;
     reader->point = reader->offset;
     reader->scale = MAX(
         ((float) data->winsize.width + reader->point.x) / ((float) reader->src.cols),
@@ -1464,7 +1478,7 @@ void icvGetAuxImages( CvMat* img, CvMat* sum, CvMat* tilted,
              + ((sqsum_type*) (sqsum->data.ptr))[p3];
 
     /* sqrt( valsqsum / area - ( valsum / are )^2 ) * area */
-    (*normfactor) = (float) sqrt( (double) (area * valsqsum - valsum * valsum) );
+    (*normfactor) = (float) sqrt( (double) (area * valsqsum - (double)valsum * valsum) );
 }
 
 CV_IMPL
@@ -1533,12 +1547,21 @@ int icvGetHaarTrainingData( CvHaarTrainingData* data, int first, int count,
     return getcount;
 }
 
+/* consumed counter */
+typedef uint64 ccounter_t;
+
+#define CCOUNTER_MAX CV_BIG_UINT(0xffffffffffffffff)
+#define CCOUNTER_SET_ZERO(cc) ((cc) = 0)
+#define CCOUNTER_INC(cc) ( (CCOUNTER_MAX > (cc) ) ? (++(cc)) : (CCOUNTER_MAX) )
+#define CCOUNTER_ADD(cc0, cc1) ( ((CCOUNTER_MAX-(cc1)) > (cc0) ) ? ((cc0) += (cc1)) : ((cc0) = CCOUNTER_MAX) )
+#define CCOUNTER_DIV(cc0, cc1) ( ((cc1) == 0) ? 0 : ( ((double)(cc0))/(double)(int64)(cc1) ) )
+
 CV_IMPL
 int icvGetHaarTrainingDataFromBG( CvHaarTrainingData* data, int first, int count,
-                                  CvIntHaarClassifier* cascade, int* consumed )
+                                  CvIntHaarClassifier* cascade, double* acceptance_ratio )
 {
     int i = 0;
-    int consumedcount = 0;
+    ccounter_t consumed_count;
 
     /* private variables */
     CvMat img;
@@ -1549,6 +1572,8 @@ int icvGetHaarTrainingDataFromBG( CvHaarTrainingData* data, int first, int count
     sum_type* sumdata;
     sum_type* tilteddata;
     float*    normfactor;
+
+    ccounter_t thread_consumed_count;
     /* end private variables */
 
     assert( data != NULL );
@@ -1557,17 +1582,20 @@ int icvGetHaarTrainingDataFromBG( CvHaarTrainingData* data, int first, int count
 
     if( !cvbgdata ) return 0;
 
-    consumedcount = 0;
-    
+    CCOUNTER_SET_ZERO(consumed_count);
+    CCOUNTER_SET_ZERO(thread_consumed_count);
+
     #ifdef _OPENMP
-    #pragma omp parallel reduction(+: consumedcount) private(img, sum, tilted, sqsum, \
-                                                         sumdata, tilteddata, normfactor)
+    #pragma omp parallel private(img, sum, tilted, sqsum, sumdata, tilteddata, \
+                                 normfactor, thread_consumed_count)
     #endif /* _OPENMP */
     {
         sumdata    = NULL;
         tilteddata = NULL;
         normfactor = NULL;
 
+        CCOUNTER_SET_ZERO(thread_consumed_count);
+
         img = cvMat( data->winsize.height, data->winsize.width, CV_8UC1,
             cvAlloc( sizeof( uchar ) * data->winsize.height * data->winsize.width ) );
         sum = cvMat( data->winsize.height + 1, data->winsize.width + 1,
@@ -1587,7 +1615,9 @@ int icvGetHaarTrainingDataFromBG( CvHaarTrainingData* data, int first, int count
             for( ; ; )
             {
                 icvGetBackgroundImage( cvbgdata, cvbgreader, &img );
-                consumedcount++;
+                
+                CCOUNTER_INC(thread_consumed_count);
+
                 sumdata = (sum_type*) (data->sum.data.ptr + i * data->sum.step);
                 tilteddata = (sum_type*) (data->tilted.data.ptr + i * data->tilted.step);
                 normfactor = data->normfactor.data.fl + i;
@@ -1612,10 +1642,22 @@ int icvGetHaarTrainingDataFromBG( CvHaarTrainingData* data, int first, int count
 
         cvFree( (void**) &(img.data.ptr) );
         cvFree( (void**) &(sqsum.data.ptr) );
+
+        #ifdef _OPENMP
+        #pragma omp critical (c_consumed_count)
+        #endif /* _OPENMP */
+        {
+            /* consumed_count += thread_consumed_count; */
+            CCOUNTER_ADD(consumed_count, thread_consumed_count);
+        }
     } /* omp parallel */
 
-    if( consumed != NULL ) (*consumed) = consumedcount;
-
+    if( acceptance_ratio != NULL )
+    {
+        /* *acceptance_ratio = ((double) count) / consumed_count; */
+        *acceptance_ratio = CCOUNTER_DIV(count, consumed_count);
+    }
+    
     return count;
 }
 
@@ -1724,6 +1766,7 @@ void cvCreateCascadeClassifier( const char* dirname,
     int poscount = 0;
     int negcount = 0;
     int consumed = 0;
+    double false_alarm = 0;
     char stagename[PATH_MAX];
     float posweight = 1.0F;
     float negweight = 1.0F;
@@ -1803,10 +1846,9 @@ void cvCreateCascadeClassifier( const char* dirname,
 #endif /* CV_VERBOSE */
 
             negcount = icvGetHaarTrainingDataFromBG( data, poscount, nneg,
-                (CvIntHaarClassifier*) cascade, &consumed );
+                (CvIntHaarClassifier*) cascade, &false_alarm );
 #ifdef CV_VERBOSE
-            printf( "NEG: %d %d %f\n", negcount, consumed,
-                    ((float) negcount) / consumed );
+            printf( "NEG: %d %g\n", negcount, false_alarm );
             printf( "BACKGROUND PROCESSING TIME: %.2f\n",
                 (proctime + TIME( 0 )) );
 #endif /* CV_VERBOSE */
@@ -1880,6 +1922,21 @@ void cvCreateCascadeClassifier( const char* dirname,
         }
         icvReleaseIntHaarFeatures( &haar_features );
         icvReleaseHaarTrainingData( &data );
+
+        if( i == nstages )
+        {
+            char xml_path[1024];
+            int len = strlen(dirname);
+            CvHaarClassifierCascade* cascade = 0;
+            strcpy( xml_path, dirname );
+            if( xml_path[len-1] == '\\' || xml_path[len-1] == '/' )
+                len--;
+            strcpy( xml_path + len, ".xml" );
+            cascade = cvLoadHaarClassifierCascade( dirname, cvSize(winwidth,winheight) );
+            if( cascade )
+                cvSave( xml_path, cascade );
+            cvReleaseHaarClassifierCascade( &cascade );
+        }
     }
     else
     {
@@ -2067,10 +2124,11 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
     int poscount;
     int negcount;
     int consumed;
+    double false_alarm;
     double proctime;
 
     int nleaves;
-    float required_leaf_fa_rate;
+    double required_leaf_fa_rate;
     float neg_ratio;
 
     int max_clusters;
@@ -2079,9 +2137,9 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
     neg_ratio = (float) nneg / npos;
 
     nleaves = 1 + MAX( 0, maxtreesplits );
-    required_leaf_fa_rate = powf( maxfalsealarm, (float) nstages ) / nleaves;
+    required_leaf_fa_rate = pow( (double) maxfalsealarm, (double) nstages ) / nleaves;
 
-    printf( "Required leaf false alarm rate: %f\n", required_leaf_fa_rate );
+    printf( "Required leaf false alarm rate: %g\n", required_leaf_fa_rate );
 
     total_splits = 0;
 
@@ -2128,11 +2186,11 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
             {                
                 int best_clusters; /* best selected number of clusters */
                 float posweight, negweight;
-                float leaf_fa_rate;
+                double leaf_fa_rate;
 
                 if( parent ) sprintf( buf, "%d", parent->idx );
                 else sprintf( buf, "NULL" );
-	            printf( "\nParent node: %s\n\n", buf );
+                printf( "\nParent node: %s\n\n", buf );
 
                 printf( "*** 1 cluster ***\n" );
 
@@ -2156,20 +2214,25 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
 
                 nneg = (int) (neg_ratio * poscount);
                 negcount = icvGetHaarTrainingDataFromBG( training_data, poscount, nneg,
-                    (CvIntHaarClassifier*) tcc, &consumed );
+                    (CvIntHaarClassifier*) tcc, &false_alarm );
+                printf( "NEG: %d %g\n", negcount, false_alarm );
 
-                printf( "NEG: %d %d %f\n", negcount, consumed, ((float) negcount)/consumed );
                 printf( "BACKGROUND PROCESSING TIME: %.2f\n", (proctime + TIME( 0 )) );
 
                 if( negcount <= 0 )
                     CV_ERROR( CV_StsError, "Unable to obtain negative samples" );
 
-                leaf_fa_rate = (float) negcount / consumed;
+                leaf_fa_rate = false_alarm;
                 if( leaf_fa_rate <= required_leaf_fa_rate )
                 {
                     printf( "Required leaf false alarm rate achieved. "
                             "Branch training terminated.\n" );
                 }
+                else if( nleaves == 1 && tcc->next_idx == nstages )
+                {
+                    printf( "Required number of stages achieved. "
+                            "Branch training terminated.\n" );
+                }
                 else
                 {
                     CvTreeCascadeNode* single_cluster;
@@ -2212,8 +2275,7 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
                     
                     if( maxtreesplits >= 0 )
                     {
-                        max_clusters = CV_MIN( max_clusters,
-                                               maxtreesplits - total_splits + 1 );
+                        max_clusters = MIN( max_clusters, maxtreesplits - total_splits + 1 );
                     }
 
                     /* try clustering */
@@ -2439,7 +2501,7 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
 
                 if( parent ) sprintf( buf, "%d", parent->idx );
                 else sprintf( buf, "NULL" );
-	            printf( "\nParent node: %s\n", buf );
+                printf( "\nParent node: %s\n", buf );
                 printf( "Chosen number of splits: %d\n\n", (last_split->multiple_clusters)
                     ? (last_split->num_clusters - 1) : 0 );
                 
@@ -2476,10 +2538,10 @@ void cvCreateTreeCascadeClassifier( const char* dirname,
     proctime = -TIME( 0 );
 
     negcount = icvGetHaarTrainingDataFromBG( training_data, poscount, nneg,
-        (CvIntHaarClassifier*) tcc, &consumed );
+        (CvIntHaarClassifier*) tcc, &false_alarm );
+
+    printf( "NEG: %d %g\n", negcount, false_alarm );
 
-    printf( "NEG: %d %d %f\n", negcount, consumed, 
-        (consumed > 0) ? (((float) negcount)/consumed) : 0 );
     printf( "BACKGROUND PROCESSING TIME: %.2f\n", (proctime + TIME( 0 )) );
 
     if( negcount <= 0 )
