@@ -44,8 +44,6 @@
 
 /////////////////////////////////////////////////////////////////////////////////////////
 
-#define CV_SWAP( a, b, t ) ((t) = (a), (a) = (b), (b) = (t))
-
 #define icvGivens_64f( n, x, y, c, s ) \
 {                                      \
     int _i;                            \
@@ -61,6 +59,8 @@
     }                                  \
 }
 
+
+/* y[0:m,0:n] += diag(a[0:1,0:m]) * x[0:m,0:n] */
 static  void
 icvMatrAXPY_64f( int m, int n, const double* x, int dx,
                  const double* a, double* y, int dy )
@@ -88,6 +88,8 @@ icvMatrAXPY_64f( int m, int n, const double* x, int dx,
 }
 
 
+/* y[1:m,-1] = h*y[1:m,0:n]*x[0:1,0:n]'*x[-1]  (this is used for U&V reconstruction)
+   y[1:m,0:n] += h*y[1:m,0:n]*x[0:1,0:n]'*x[0:1,0:n] */
 static void
 icvMatrAXPY3_64f( int m, int n, const double* x, int l, double* y, double h )
 {
@@ -201,6 +203,7 @@ icvMatrAXPY3_32f( int m, int n, const float* x, int l, float* y, double h )
     }
 }
 
+/* accurate hypotenuse calculation */
 static double
 pythag( double a, double b )
 {
@@ -259,6 +262,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
     m1 = m;
     n1 = n;
 
+    /* transform a to bi-diagonal form */
     for( ;; )
     {
         int update_u;
@@ -390,7 +394,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
 
         for( i = m1 - 1; i >= 0; i-- )
         {
-            double h, s;
+            double s;
             int lh = nu - i;
 
             l = m - i;
@@ -434,7 +438,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
 
         for( i = n1 - 1; i >= 0; i-- )
         {
-            double h, s;
+            double s;
             int lh = nv - i;
 
             l = n - i;
@@ -483,7 +487,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
 
         for( ;; )               /* do iterations */
         {
-            double c, s, f, g, h, x, y;
+            double c, s, f, g, x, y;
             int flag = 0;
 
             /* test for splitting */
@@ -506,7 +510,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
 
                 for( i = l; i <= k; i++ )
                 {
-                    double f = s * e[i];
+                    f = s * e[i];
 
                     e[i] *= c;
 
@@ -520,7 +524,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
                     s = -f / h;
 
                     if( uT )
-                        icvGivens_64f( m, uT + lduT * (i - 1), uT + lduT * i, c, s );
+                        icvGivens_64f( m, uT + lduT * (l - 1), uT + lduT * i, c, s );
                 }
             }
 
@@ -594,7 +598,7 @@ icvSVD_64f( double* a, int lda, int m, int n,
         }
     }                           /* end of diagonalization loop */
 
-    /* sort singular values */
+    /* sort singular values and corresponding values */
     for( i = 0; i < nm; i++ )
     {
         k = i;
@@ -604,7 +608,6 @@ icvSVD_64f( double* a, int lda, int m, int n,
 
         if( k != i )
         {
-            /* swap i & k values */
             double t;
             CV_SWAP( w[i], w[k], t );
 
@@ -655,6 +658,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
     m1 = m;
     n1 = n;
 
+    /* transform a to bi-diagonal form */
     for( ;; )
     {
         int update_u;
@@ -787,7 +791,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
 
         for( i = m1 - 1; i >= 0; i-- )
         {
-            double h, s;
+            double s;
             int lh = nu - i;
 
             l = m - i;
@@ -831,7 +835,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
 
         for( i = n1 - 1; i >= 0; i-- )
         {
-            double h, s;
+            double s;
             int lh = nv - i;
 
             l = n - i;
@@ -880,7 +884,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
 
         for( ;; )               /* do iterations */
         {
-            double c, s, f, g, h, x, y;
+            double c, s, f, g, x, y;
             int flag = 0;
 
             /* test for splitting */
@@ -903,8 +907,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
 
                 for( i = l; i <= k; i++ )
                 {
-                    double f = s * e[i];
-
+                    f = s * e[i];
                     e[i] = (float)(e[i]*c);
 
                     if( anorm + fabs( f ) == anorm )
@@ -917,7 +920,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
                     s = -f / h;
 
                     if( uT )
-                        icvGivens_32f( m, uT + lduT * (i - 1), uT + lduT * i, c, s );
+                        icvGivens_32f( m, uT + lduT * (l - 1), uT + lduT * i, c, s );
                 }
             }
 
@@ -991,7 +994,7 @@ icvSVD_32f( float* a, int lda, int m, int n,
         }
     }                           /* end of diagonalization loop */
 
-    /* sort singular values */
+    /* sort singular values and corresponding vectors */
     for( i = 0; i < nm; i++ )
     {
         k = i;
@@ -1001,7 +1004,6 @@ icvSVD_32f( float* a, int lda, int m, int n,
 
         if( k != i )
         {
-            /* swap i & k values */
             float t;
             CV_SWAP( w[i], w[k], t );
 
@@ -1224,8 +1226,8 @@ cvSVD( CvArr* aarr, CvArr* warr, CvArr* uarr, CvArr* varr, int flags )
     uchar* tw = 0;
     int type;
     int a_buf_offset = 0, u_buf_offset = 0, buf_size, pix_size;
-    int temp_u = 0, // temporary storage for U is needed
-        t_svd; // special case: a->rows < a->cols
+    int temp_u = 0, /* temporary storage for U is needed */
+        t_svd; /* special case: a->rows < a->cols */
     int m, n;
     int w_rows, w_cols;
     int u_rows = 0, u_cols = 0;
@@ -1290,12 +1292,14 @@ cvSVD( CvArr* aarr, CvArr* warr, CvArr* uarr, CvArr* varr, int flags )
             CV_ERROR( CV_StsUnmatchedFormats, "" );
 
         if( u_rows != m || (u_cols != m && u_cols != n))
-            CV_ERROR( CV_StsUnmatchedSizes, "U matrix has unappropriate size" );
+            CV_ERROR( CV_StsUnmatchedSizes, !t_svd ? "U matrix has unappropriate size" :
+                                                     "V matrix has unappropriate size" );
             
         temp_u = (u_rows != u_cols && !(flags & CV_SVD_U_T)) || u->data.ptr==a->data.ptr;
 
         if( w_is_mat && u_cols != w_rows )
-            CV_ERROR( CV_StsUnmatchedSizes, "U and W have incompatible sizes" );
+            CV_ERROR( CV_StsUnmatchedSizes, !t_svd ? "U and W have incompatible sizes" :
+                                                     "V and W have incompatible sizes" );
     }
     else
     {
@@ -1326,10 +1330,12 @@ cvSVD( CvArr* aarr, CvArr* warr, CvArr* uarr, CvArr* varr, int flags )
             CV_ERROR( CV_StsUnmatchedFormats, "" );
 
         if( v_rows != n || v_cols != n )
-            CV_ERROR( CV_StsUnmatchedSizes, "V matrix has unappropriate size" );
+            CV_ERROR( CV_StsUnmatchedSizes, t_svd ? "U matrix has unappropriate size" :
+                                                    "V matrix has unappropriate size" );
 
         if( w_is_mat && w_cols != v_cols )
-            CV_ERROR( CV_StsUnmatchedSizes, "V and W have incompatible sizes" );
+            CV_ERROR( CV_StsUnmatchedSizes, t_svd ? "U and W have incompatible sizes" :
+                                                    "V and W have incompatible sizes" );
     }
     else
     {
