@@ -46,7 +46,7 @@
 
 #include "_cxcore.h"
 
-#if _MSC_VER >= 1200
+#if defined _MSC_VER && _MSC_VER >= 1200
 #pragma warning( disable: 4115 )        /* type definition in () */
 #endif
 
@@ -68,18 +68,18 @@
 #define CV_PROC_IA32_PII            (CV_PROC_IA32_GENERIC|(2 << CV_PROC_SHIFT))
 #define CV_PROC_IA32_PIII           (CV_PROC_IA32_GENERIC|(3 << CV_PROC_SHIFT))
 #define CV_PROC_IA32_P4             (CV_PROC_IA32_GENERIC|(4 << CV_PROC_SHIFT))
-#define CV_PROC_IA64_GENERIC        2
+#define CV_PROC_IA64_ITANIUM        2
+#define CV_PROC_IA64_EM64T          3
 #define CV_GET_PROC_ARCH(model)     ((model) & CV_PROC_ARCH_MASK)
 
 typedef struct CvProcessorInfo
 {
     int model;
     int count;
-    double frequency; // clocks per microseconds
+    double frequency; // clocks per microsecond
 }
 CvProcessorInfo;
 
-
 /*
    determine processor type
 */
@@ -90,58 +90,107 @@ icvInitProcessorInfo( CvProcessorInfo* cpu_info )
     cpu_info->model = CV_PROC_GENERIC;
 
 #if defined WIN32 || defined WIN64
+
+#ifndef PROCESSOR_ARCHITECTURE_AMD64
+#define PROCESSOR_ARCHITECTURE_AMD64 9
+#endif
+
+#ifndef PROCESSOR_ARCHITECTURE_IA32_ON_WIN64
+#define PROCESSOR_ARCHITECTURE_IA32_ON_WIN64 10
+#endif
+
     SYSTEM_INFO sys;
+    LARGE_INTEGER freq;
+
     GetSystemInfo( &sys );
 
     if( sys.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL &&
-        sys.dwProcessorType == PROCESSOR_INTEL_PENTIUM )
+        sys.dwProcessorType == PROCESSOR_INTEL_PENTIUM && sys.wProcessorLevel >= 6 )
     {
-        static const char cpuid_code[] =
-            "\x53\x56\x57\xb8\x01\x00\x00\x00\x0f\xa2\x5f\x5e\x5b\xc3";
-        typedef int64 (CV_CDECL * func_ptr)(void);
-        func_ptr cpuid = (func_ptr)(void*)cpuid_code;
         int version = 0, features = 0, family = 0;
         int id = 0;
-        int64 cpuid_val;
         HKEY key = 0;
-        
-        cpu_info->count = (int)sys.dwNumberOfProcessors; 
-        unsigned long freq = 0, sz = sizeof(freq);
+
+        cpu_info->count = (int)sys.dwNumberOfProcessors;
+        unsigned long val = 0, sz = sizeof(val);
 
         if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\SYSTEM\\CentralProcessor\\0\\",
             0, KEY_QUERY_VALUE, &key ) >= 0 )
         {
-            if( RegQueryValueEx( key, "~MHz", 0, 0, (uchar*)&freq, &sz ) >= 0 )
-                cpu_info->frequency = (double)freq;
+            if( RegQueryValueEx( key, "~MHz", 0, 0, (uchar*)&val, &sz ) >= 0 )
+                cpu_info->frequency = (double)val;
             RegCloseKey( key );
         }
 
-        cpuid_val = cpuid();
-        version = (int)cpuid_val;
-        features = (int)(cpuid_val >> 32);
+#if defined WIN32 && !defined WIN64 && (defined _MSC_VER || defined __BORLANDC__ && __BORLANDC__>=0x560)
+        __asm
+        {
+            /* use CPUID to determine the features supported */
+            pushfd
+            mov   eax, 1
+            push  ebx
+            push  esi
+            push  edi
+#ifdef __BORLANDC__
+            db 0fh
+            db 0a2h
+#else
+            _emit 0x0f
+            _emit 0xa2
+#endif
+            pop   edi
+            pop   esi
+            pop   ebx
+            mov   version, eax
+            mov   features, edx
+            popfd
+        }
+#elif defined WIN32 && __GNUC__ > 2
+        asm volatile
+        (
+            "movl $1,%%eax\n\t"
+            ".byte 0x0f; .byte 0xa2\n\t"
+            "movl %%eax, %0\n\t"
+            "movl %%edx, %1\n\t"
+            : "=r"(version), "=r" (features)
+            :
+            : "%ebx", "%esi", "%edi"
+        );
+#else
+        {
+            static const char cpuid_code[] =
+                "\x53\x56\x57\xb8\x01\x00\x00\x00\x0f\xa2\x5f\x5e\x5b\xc3";
+            typedef int64 (CV_CDECL * func_ptr)(void);
+            func_ptr cpuid = (func_ptr)(void*)cpuid_code;
+            int64 cpuid_val = cpuid();
+            version = (int)cpuid_val;
+            features = (int)(cpuid_val >> 32);
+        }
+#endif
 
-        #define ICV_CPUID_M6     ((1<<15)|(1<<23)|6)  /* cmov + mmx */
-        #define ICV_CPUID_A6     ((1<<25)|ICV_CPUID_M6) /* <all above> + xmm */
-        #define ICV_CPUID_W7     ((1<<26)|ICV_CPUID_A6|(1<<3)|1) /* <all above> + emm */
+        #define ICV_CPUID_M6     ((1<<15)|(1<<23))  /* cmov + MMX */
+        #define ICV_CPUID_A6     ((1<<25)|ICV_CPUID_M6) /* <all above> + SSE */
+        #define ICV_CPUID_W7     ((1<<26)|ICV_CPUID_A6) /* <all above> + SSE2 */
 
         family = (version >> 8) & 15;
-        if( family >= 6 && (features & (ICV_CPUID_M6 & ~6)) != 0 ) /* Pentium II or higher */
-            id = (features & ICV_CPUID_W7 & -256) | family;
+        if( family >= 6 && (features & ICV_CPUID_M6) != 0 ) /* Pentium II or higher */
+            id = features & ICV_CPUID_W7;
 
         cpu_info->model = id == ICV_CPUID_W7 ? CV_PROC_IA32_P4 :
-                          id == ICV_CPUID_A6 ? CV_PROC_IA32_PIII : 
+                          id == ICV_CPUID_A6 ? CV_PROC_IA32_PIII :
                           id == ICV_CPUID_M6 ? CV_PROC_IA32_PII :
                           CV_PROC_IA32_GENERIC;
     }
     else
     {
-        LARGE_INTEGER freq;
-#ifdef WIN64
-        if( sys.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL &&
-            sys.dwProcessorType == PROCESSOR_ARCHITECTURE_IA64 )
-            cpu_info->model = CV_PROC_IA64_GENERIC;
+#if defined EM64T
+        if( sys.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64 )
+            cpu_info->model = CV_PROC_IA64_EM64T;
+#elif defined WIN64
+        if( sys.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64 )
+            cpu_info->model = CV_PROC_IA64_ITANIUM;
 #endif
-        if( QueryPerformanceFrequency( &freq ) ) 
+        if( QueryPerformanceFrequency( &freq ) )
             cpu_info->frequency = (double)freq.QuadPart;
     }
 #else
@@ -250,7 +299,7 @@ typedef void* HMODULE;
 #define VERBOSE_LOADING 0
 
 #if VERBOSE_LOADING
-#define ICV_PRINTF(args)  printf args
+#define ICV_PRINTF(args)  printf args; fflush(stdout)
 #else
 #define ICV_PRINTF(args)
 #endif
@@ -264,7 +313,7 @@ typedef struct CvPluginInfo
 CvPluginInfo;
 
 static CvPluginInfo plugins[CV_PLUGIN_MAX];
-static CvModuleInfo cxcore_module = { 0, "cxcore", "beta 4.0", cxcore_ipp_tab };
+static CvModuleInfo cxcore_module = { 0, "cxcore", "beta 5 (0.9.7)", cxcore_ipp_tab };
 
 static CvModuleInfo *icvFirstModule = 0, *icvLastModule = 0;
 
@@ -272,7 +321,7 @@ static int
 icvUpdatePluginFuncTab( CvPluginFuncInfo* func_tab )
 {
     int i, loaded_functions = 0;
-    
+
     // 1. reset pointers
     for( i = 0; func_tab[i].func_addr != 0; i++ )
     {
@@ -283,10 +332,20 @@ icvUpdatePluginFuncTab( CvPluginFuncInfo* func_tab )
         func_tab[i].loaded_from = 0;
     }
 
+    // ippopencv substitutes all the other IPP modules
+    if( plugins[CV_PLUGIN_OPTCV].handle != 0 )
+    {
+        for( i = 2; i < CV_PLUGIN_MKL; i++ )
+        {
+            assert( plugins[i].handle == 0 );
+            plugins[i].handle = plugins[CV_PLUGIN_OPTCV].handle;
+        }
+    }
+
     // 2. try to find corresponding functions in ipp* and reassign pointers to them
     for( i = 0; func_tab[i].func_addr != 0; i++ )
     {
-    #if _MSC_VER >= 1200
+    #if defined _MSC_VER && _MSC_VER >= 1200
         #pragma warning( disable: 4054 4055 ) /* converting pointers to code<->data */
     #endif
         char name[100];
@@ -340,12 +399,12 @@ icvUpdatePluginFuncTab( CvPluginFuncInfo* func_tab )
                 ICV_PRINTF(("%s: \t%s\n", name, plugins[idx].name ));
             }
 
-            #if _MSC_VER >= 1200
+            #if defined _MSC_VER && _MSC_VER >= 1200
                 #pragma warning( default: 4054 4055 )
             #endif
         }
     }
-    
+
 #if VERBOSE_LOADING
     {
     int not_loaded = 0;
@@ -362,6 +421,12 @@ icvUpdatePluginFuncTab( CvPluginFuncInfo* func_tab )
     }
 #endif
 
+    if( plugins[CV_PLUGIN_OPTCV].handle != 0 )
+    {
+        for( i = 2; i < CV_PLUGIN_MKL; i++ )
+            plugins[i].handle = 0;
+    }
+
     return loaded_functions;
 }
 
@@ -370,7 +435,7 @@ CV_IMPL int
 cvRegisterModule( const CvModuleInfo* module )
 {
     CvModuleInfo* module_copy = 0;
-    
+
     CV_FUNCNAME( "cvRegisterModule" );
 
     __BEGIN__;
@@ -424,59 +489,71 @@ cvUseOptimized( int load_flag )
     CvModuleInfo* module;
     const CvProcessorInfo* cpu_info = icvGetProcessorInfo();
     int arch = CV_GET_PROC_ARCH(cpu_info->model);
+    const char* opencv_suffix = "097";
     const char* ipp_suffix = arch == CV_PROC_IA32_GENERIC ? "20" :
-                             arch == CV_PROC_IA64_GENERIC ? "6420" : 0;
+                             arch == CV_PROC_IA64_ITANIUM ? "6420" :
+                             arch == CV_PROC_IA64_EM64T ? "em64t" : "";
     const char* mkl_suffix = arch == CV_PROC_IA32_GENERIC ?
                 (cpu_info->model >= CV_PROC_IA32_P4 ? "p4" :
                  cpu_info->model >= CV_PROC_IA32_PIII ? "p3" : "def") :
-                 arch == CV_PROC_IA64_GENERIC ? "itp" : 0;
+                 arch == CV_PROC_IA64_ITANIUM ? "itp" : "";
 
     for( i = 0; i < CV_PLUGIN_MAX; i++ )
         plugins[i].basename = 0;
     plugins[CV_PLUGIN_NONE].basename = 0;
     plugins[CV_PLUGIN_NONE].name[0] = '\0';
-    plugins[CV_PLUGIN_OPTCV].basename = "optcv";
+    plugins[CV_PLUGIN_OPTCV].basename = "ippopencv";
     plugins[CV_PLUGIN_IPPCV].basename = "ippcv";
     plugins[CV_PLUGIN_IPPI].basename = "ippi";
     plugins[CV_PLUGIN_IPPS].basename = "ipps";
     plugins[CV_PLUGIN_IPPVM].basename = "ippvm";
+    plugins[CV_PLUGIN_IPPCC].basename = "ippcc";
     plugins[CV_PLUGIN_MKL].basename = "mkl_";
 
-    if( ipp_suffix )
+    // try to load optimized dlls
+    for( i = 1; i < CV_PLUGIN_MAX; i++ )
     {
-        // try to load optimized dlls
-        for( i = 1; i < CV_PLUGIN_MAX; i++ )
+        // unload previously loaded optimized modules
+        if( plugins[i].handle )
         {
-            // unload previously loaded optimized modules
-            if( plugins[i].handle )
-            {
-                FreeLibrary( plugins[i].handle );
-                plugins[i].handle = 0;
-            }
+            FreeLibrary( plugins[i].handle );
+            plugins[i].handle = 0;
+        }
 
-            if( load_flag && plugins[i].basename &&
-                (arch == CV_PROC_IA32_GENERIC || arch == CV_PROC_IA64_GENERIC) )
+        // do not load regular IPP modules if the custom merged IPP module is already found.
+        if( i < CV_PLUGIN_MKL && load_flag && plugins[CV_PLUGIN_OPTCV].handle != 0 )
+            continue;
+
+        if( load_flag && plugins[i].basename &&
+            (arch == CV_PROC_IA32_GENERIC || arch == CV_PROC_IA64_ITANIUM || arch == CV_PROC_IA64_EM64T) )
+        {
+            const char* suffix = i == CV_PLUGIN_OPTCV ? opencv_suffix :
+                            i < CV_PLUGIN_MKL ? ipp_suffix : mkl_suffix;
+            for(;;)
             {
-                for(;;)
+                sprintf( plugins[i].name, DLL_PREFIX "%s%s" DLL_DEBUG_FLAG DLL_SUFFIX,
+                    plugins[i].basename, suffix );
+
+                ICV_PRINTF(("loading %s...\n", plugins[i].name ));
+                plugins[i].handle = LoadLibrary( plugins[i].name );
+                if( plugins[i].handle != 0 )
                 {
-                    sprintf( plugins[i].name, DLL_PREFIX "%s%s" DLL_DEBUG_FLAG DLL_SUFFIX,
-                             plugins[i].basename, i < CV_PLUGIN_MKL ? ipp_suffix : mkl_suffix );
-                
-                    plugins[i].handle = LoadLibrary( plugins[i].name );
-                    if( plugins[i].handle != 0 )
-                    {
-                        ICV_PRINTF(("%s loaded\n", plugins[i].name )); 
-                        loaded_modules++;
-                    }
-                    if( plugins[i].handle != 0 || i < CV_PLUGIN_MKL )
-                        break;
-                    if( strcmp( mkl_suffix, "p4" ) == 0 )
-                        mkl_suffix = "p3";
-                    else if( strcmp( mkl_suffix, "p3" ) == 0 )
-                        mkl_suffix = "def";
-                    else
-                        break;
+                    ICV_PRINTF(("%s loaded\n", plugins[i].name ));
+                    loaded_modules++;
                 }
+                if( plugins[i].handle != 0 )
+                    break;
+
+                if( strcmp( suffix, "p4" ) == 0 )
+                    suffix = "p3";
+                else if( strcmp( suffix, "p3" ) == 0 )
+                    suffix = "def";
+                else if( strcmp( suffix, "20" ) == 0 )
+                    suffix = "";
+                else if( strcmp( suffix, "6420" ) == 0 )
+                    suffix = "64";
+                else
+                    break;
             }
         }
     }
@@ -546,13 +623,13 @@ cvGetModuleInfo( const char* name, const char **version, const char **plugin_lis
             *version = joint_verinfo;
         }
     }
-    
+
     if( plugin_list )
     {
         char* ptr = plugin_list_buf;
         int i;
 
-        for( i = 0; plugins[i].basename != 0; i++ )
+        for( i = 0; i < CV_PLUGIN_MAX; i++ )
             if( plugins[i].handle != 0 )
             {
                 int n = 0;
@@ -582,9 +659,23 @@ CV_IMPL  int64  cvGetTickCount( void )
 
     if( CV_GET_PROC_ARCH(cpu_info->model) == CV_PROC_IA32_GENERIC )
     {
+#if defined WIN32 && !defined WIN64 && (defined _MSC_VER || defined __BORLANDC__ && __BORLANDC__>=0x560)
+    #ifdef __BORLANDC__
+        __asm db 0fh
+        __asm db 31h
+    #else
+        __asm _emit 0x0f;
+        __asm _emit 0x31;
+    #endif
+#elif (defined __GNUC__ || defined CV_ICC) && defined __i386__
+        int64 t;
+        asm volatile (".byte 0xf; .byte 0x31" /* "rdtsc" */ : "=A" (t));
+        return t;
+#else
         static const char code[] = "\x0f\x31\xc3";
         rdtsc_func func = (rdtsc_func)(void*)code;
         return func();
+#endif
     }
     else
     {
@@ -593,8 +684,8 @@ CV_IMPL  int64  cvGetTickCount( void )
         QueryPerformanceCounter( &counter );
         return (int64)counter.QuadPart;
 #else
-        timeval tv;
-        timezone tz;
+        struct timeval tv;
+        struct timezone tz;
         gettimeofday( &tv, &tz );
         return (int64)tv.tv_sec*1000000 + tv.tv_usec;
 #endif
